/***********************************************************************************************************************
 * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
 * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
 * sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for the selection and use
 * of Renesas products and Renesas assumes no liability.  No license, express or implied, to any intellectual property
 * right is granted by Renesas. This software is protected under all applicable laws, including copyright laws. Renesas
 * reserves the right to change or discontinue this software and/or this documentation. THE SOFTWARE AND DOCUMENTATION
 * IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND TO THE FULLEST EXTENT
 * PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY, INCLUDING WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE SOFTWARE OR
 * DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.  TO THE MAXIMUM
 * EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR DOCUMENTATION
 * (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER, INCLUDING,
 * WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY LOST PROFITS,
 * OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/
#include <sample_main.h>
#include "hal_data.h"
#include "da16200_AT.h"
#include "common_utils.h"

/** AT Command sets */
/*LDRA_INSPECTED 27 D This structure must be accessible in user code. It cannot be static. */
da16200_at_cmd_set_t g_da16200_cmd_set[] =
{
    /* Intial AT command */
    [DA16200_AT_CMD_INDEX_ATZ] =
    {
        .p_cmd = (uint8_t *) "ATZ\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_64,
        .retry = DA16200_RETRY_VALUE_10,
        .retry_delay = DA16200_DELAY_500MS
    },
    /* Echo on/ off */
    [DA16200_AT_CMD_INDEX_ATE] =
    {
        .p_cmd = (uint8_t *) "ATE\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_32,
        .retry = DA16200_RETRY_VALUE_5,
        .retry_delay = DA16200_DELAY_200MS
    },

    /* Set Station mode */
    [DA16200_AT_CMD_INDEX_AT_WFMODE_STA] =
    {
        .p_cmd = (uint8_t *) "AT+WFMODE=0\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_32,
        .retry = DA16200_RETRY_VALUE_5,
        .retry_delay = DA16200_DELAY_200MS
    },
    /* Set Country Code */
    [DA16200_AT_CMD_INDEX_AT_WFCC] =
    {
        .p_cmd = (uint8_t *) "AT+WFCC=CH\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_32,
        .retry = DA16200_RETRY_VALUE_5,
        .retry_delay = DA16200_DELAY_200MS
    },
    /* Connect to a AP */
    [ DA16200_AT_CMD_INDEX_AT_WFJAP] =
    {
        //.p_cmd = (uint8_t *) "",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_128,
        .retry = DA16200_RETRY_VALUE_5,
        .retry_delay = DA16200_DELAY_200MS
    },
    /* Restart */
    [ DA16200_AT_CMD_INDEX_AT_RESTART] =
    {
        .p_cmd = (uint8_t *) "AT+RESTART\r\n",
        .p_success_resp = (uint8_t *) "DONE",
        .max_resp_length = DA16200_STR_LEN_128,
        .retry = DA16200_RETRY_VALUE_5,
        .retry_delay = DA16200_DELAY_1000MS
    },
    /* Start DHCP client*/
    [ DA16200_AT_CMD_INDEX_AT_NWDHC] =
    {
        .p_cmd = (uint8_t *) "AT+NWDHC=1\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_64,
        .retry = DA16200_RETRY_VALUE_5,
        .retry_delay = DA16200_DELAY_500MS
    },
    /* Set the HTTP server name indication */
    [ DA16200_AT_CMD_INDEX_AT_NWHTCSNI] =
    {
        //.p_cmd = (uint8_t *) "AT+NWHTCSNI httpbin.org\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_32,
        .retry = DA16200_RETRY_VALUE_1,
        .retry_delay = DA16200_DELAY_200MS
    },

    /* Set the application layer protocol negotiation */
    [ DA16200_AT_CMD_INDEX_AT_NWHTCALPN] =
    {
        .p_cmd = (uint8_t *) "AT+NWHTCALPN 1,http/1.1\r\n",
        .p_success_resp = (uint8_t *) "OK",
        .max_resp_length = DA16200_STR_LEN_32,
        .retry = DA16200_RETRY_VALUE_1,
        .retry_delay = DA16200_DELAY_500MS
    },

    /* Start the HTTP client */
    [ DA16200_AT_CMD_INDEX_AT_NWHTC] =
    {
        .p_cmd = (uint8_t *) "AT+NWHTC=http://",
        //.p_cmd = (uint8_t *) "AT+NWHTC=http://httpbin.org/post,post,HTTP Client post method sample test\r\n",
        .p_success_resp = (uint8_t *) "code:200",
        .max_resp_length = DA16200_STR_LEN_512,
        .retry = DA16200_RETRY_VALUE_1,
        .retry_delay = DA16200_DELAY_200MS
    }

};

uint8_t g_wifi_receive_complete = 0;
uint8_t g_wifi_transmit_complete = 0;
uint16_t g_wifi_receive_time_count = 0;
static uint16_t count = 0;

static fsp_err_t wifi_command_ATZ(void);
static fsp_err_t wifi_command_ATE(void);
static fsp_err_t wifi_command_AT_WFMODE_STA(void);
static fsp_err_t wifi_command_AT_RESTART(void);
static fsp_err_t wifi_command_AT_WFCC(void);
static fsp_err_t wifi_command_AT_WFJAP(void);
static fsp_err_t wifi_command_AT_NWDHC(void);
static fsp_err_t wifi_command_AT_NWHTCSNI(void);
static fsp_err_t wifi_command_AT_NWHTCALPN(void);
static fsp_err_t wifi_command_AT_NWHTC(void);

/************************************************************************************
* Name:       is_str_present
* Function:   compare string
* Parameters: p_resp_str, p_search_str
* Return:     comparing result
************************************************************************************/
uint8_t is_str_present(const char * p_resp_str, const char * p_search_str)
{
    if (strstr (p_resp_str, p_search_str))
    {
        return SF_WIFI_TRUE;
    }

    return SF_WIFI_FALSE;
}

/************************************************************************************
* Name:       wifi_command_ATZ
* Function:   Operate ATZ command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_ATZ(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_64] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * ATZ
     * initialize AT command
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_ATZ].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_ATZ].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_ATZ].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 100);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_ATZ].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }
        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_ATZ].retry_delay, BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_ATZ].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_ATE
* Function:   Operate ATE command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_ATE(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_32] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * ATE
     * Echo AT command
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_ATE].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_ATE].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_ATE].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 500);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_ATE].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_ATE].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_ATE].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_WFMODE_STA
* Function:   Operate AT+WFMODE command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_WFMODE_STA(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_32] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+WFMODE = 0
     * AT Wifi mode
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFMODE_STA].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFMODE_STA].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFMODE_STA].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 500);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFMODE_STA].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFMODE_STA].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFMODE_STA].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_WFCC
* Function:   Operate AT+WFCC command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_WFCC(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_32] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+WFCC = CH
     * AT Country Code
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFCC].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFCC].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFCC].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 500);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFCC].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFCC].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFCC].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_WFJAP
* Function:   Operate AT+WFJAP command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_WFJAP(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_128] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+WFAP = 1
     * AT Country Code
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 3000);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_RESTART
* Function:   Operate AT+RESTART command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_RESTART(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_128] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+RESTART = 1
     * AT Country Code
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_RESTART].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_RESTART].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_RESTART].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 8000);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_RESTART].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_RESTART].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_RESTART].retry);

    return retval;
}



/************************************************************************************
* Name:       wifi_command_AT_NWDHC
* Function:   Operate AT+NWDHC command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_NWDHC(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_512] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+WWDHS = 1
     * AT Country Code
     */

    do
    {
        /** AT MODODR command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWDHC].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWDHC].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWDHC].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 1000);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWDHC].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWDHC].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWDHC].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_NWHTCSNI
* Function:   Operate AT+NWHTCSNI command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_NWHTCSNI(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_32] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+NWHTCSNI
     * set the server name indication
     */

    do
    {
        /** AT NWHTCSNI command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCSNI].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCSNI].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCSNI].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 300);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCSNI].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCSNI].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCSNI].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_NWHTCLPN
* Function:   Operate AT+NWHTCLPN command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_NWHTCALPN(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_32] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+NWHTCALPN
     * set the application layer protocol negotiation
     */

    do
    {
        /** AT NWHTCLPN command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCALPN].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCALPN].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCALPN].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 300);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCALPN].p_success_resp);
            if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCALPN].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTCALPN].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_command_AT_NWHTC
* Function:   Operate AT+NWHTC command
* Parameters: none
* Return:     AT command operation status
************************************************************************************/
static fsp_err_t wifi_command_AT_NWHTC(void)
{
    uint16_t bytes_read = 0U;
    uint16_t bytes_write;
    uint8_t resp_buff[DA16200_STR_LEN_512] =
    {0};
    fsp_err_t retval = FSP_ERR_ASSERTION;
    fsp_err_t result;
    uint8_t retry_count = 0U;
    da16200_at_cmd_set_t  * p_cmd_set = g_da16200_cmd_set;
    uint8_t expected_resp_present;

    /**
     * AT+NWHTC
     * Start the HTTP client with options
     */

    do
    {
        /** AT NWHTC command **/
        bytes_write = (uint16_t) strlen((char *) p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].p_cmd);
        wifi_serial_write((uint8_t*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].p_cmd,bytes_write);

        bytes_read = p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].max_resp_length;

        /** Clear respond memory **/
        memset (resp_buff, 0 , sizeof(resp_buff));

        result = wifi_serial_read(resp_buff, &bytes_read, 3000);

        if((FSP_SUCCESS == result) || (FSP_ERR_TIMEOUT == result))
        {
            expected_resp_present = is_str_present((const char *)resp_buff, (const char*)p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].p_success_resp);
            //if(SF_WIFI_TRUE == expected_resp_present)
            {
                retval = FSP_SUCCESS;
                break;
            }
        }

        R_BSP_SoftwareDelay(p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].retry_delay,BSP_DELAY_UNITS_MILLISECONDS);

        ++retry_count;

    }
    while(retry_count < p_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].retry);

    return retval;
}

/************************************************************************************
* Name:       wifi_setting
* Function:   setting Wifi
* Parameters: none
* Return:     wifi setting status
************************************************************************************/
fsp_err_t wifi_setting(char *ssid, char *ssid_pw)
{
    fsp_err_t status = FSP_SUCCESS;
    uint8_t atcmd_buffer[100];

    status = wifi_command_ATZ();            //AT cmd reset

    status = wifi_command_ATE();            //Echo on

    status = wifi_command_AT_WFMODE_STA();  //Set wifi mode to station mode

    status = wifi_command_AT_RESTART();     //restart

    status = wifi_command_ATE();            //Echo on

    status = wifi_command_AT_WFCC();        //Set a country code to CH

    memset(atcmd_buffer, 0, 100);
    /* Joint the string for WFJAP command: AT_WFJAP=ssid,4,1,ssid_pw\r\n */
    strcat((char *)atcmd_buffer, (char *)"AT+WFJAP=");
    strcat((char *)atcmd_buffer, (char *)ssid);
    strcat((char *)atcmd_buffer, (char *)",4,1,");
    strcat((char *)atcmd_buffer, (char *)ssid_pw);
    strcat((char *)atcmd_buffer, (char *)"\r\n");
    g_da16200_cmd_set[DA16200_AT_CMD_INDEX_AT_WFJAP].p_cmd = atcmd_buffer;
    status = wifi_command_AT_WFJAP();       //Connect to a AP

    status = wifi_command_AT_NWDHC();       //Start the DHCP client

    status = wifi_command_AT_NWHTCALPN();

    if(FSP_SUCCESS != status)
    {
        APP_PRINT("Wifi setting Error! \r\n");
    }
    else
    {
        APP_PRINT("Wifi setting OK, please press S1 button to post one message \r\n");
    }

    return status;
}

/************************************************************************************
* Name:       wifi_serial_read
* Function:   read receive data
* Parameters: p_dest, p_bytes, timeout_ms
* Return:     read result
************************************************************************************/
fsp_err_t wifi_serial_read(uint8_t * p_dest, uint16_t * p_bytes, uint32_t timeout_ms)
{
    fsp_err_t status = FSP_SUCCESS;
    uint16_t bytes = *p_bytes;
    uint8_t temp_mem;
    uint16_t temp_count = 0;
    g_wifi_receive_time_count = 0;
    do
    {
          g_wifi_uart.p_api->read(g_wifi_uart.p_ctrl, &temp_mem, 1);
          if(g_wifi_receive_complete)
          {
              g_wifi_receive_complete = 0;
              *p_dest = temp_mem;
               p_dest++;
               temp_count++;
               if(temp_count >= bytes)

               {
                   break;
               }
          }
          if(g_wifi_receive_time_count >= timeout_ms)
          {
               status = FSP_ERR_TIMEOUT;
               break;
          }

    }while(1);

    return status;
}


/************************************************************************************
* Name:       wifi_serial_write
* Function:   write data
* Parameters: p_src, bytes
* Return:     write result
************************************************************************************/
fsp_err_t wifi_serial_write(uint8_t * p_src, uint16_t bytes)
{
    fsp_err_t status = FSP_SUCCESS;


    status = g_wifi_uart.p_api->write(g_wifi_uart.p_ctrl, p_src, bytes);

    return(status);

}

/************************************************************************************
* Name:       user_uart_callback
* Function:   write data callback
* Parameters: p_args
* Return:     write result
************************************************************************************/
void user_uart_callback(uart_callback_args_t *p_args)
{
    if(UART_EVENT_RX_COMPLETE == p_args->event)
    {
        g_wifi_receive_complete = 1;
    }
}

/************************************************************************************
* Name:       timer_1ms_callback
* Function:   1ms callback
* Parameters: p_args
* Return:     1ms timing
************************************************************************************/
void timer_1ms_callback(timer_callback_args_t *p_args)
{
    static uint16_t led_blink_count;

    FSP_PARAMETER_NOT_USED(p_args);

    g_wifi_receive_time_count++;

    if(led_blink_flag)
    {
        led_blink_count++;

        if(led_blink_count < 1000)
        {
            g_ioport.p_api->pinWrite(g_ioport.p_ctrl, RED_LED, BSP_IO_LEVEL_HIGH);
        }
        else if((led_blink_count < 2000)&&(led_blink_count >= 1000))
        {
            g_ioport.p_api->pinWrite(g_ioport.p_ctrl, RED_LED, BSP_IO_LEVEL_LOW);
        }
        else if(led_blink_count > 2000)
        {
            led_blink_count = 0;
        }
    }
}

/************************************************************************************
* Name:       wifi_http_client_send_data
* Function:   1ms callback
* Parameters: p_args
* Return:     1ms timing
************************************************************************************/
fsp_err_t wifi_http_client_send_data(char *server_ip, char *data_to_send,uint8_t num)
{
    fsp_err_t status = FSP_SUCCESS;
    uint8_t atcmd_buffer[MAX_LEN_TRANSMIT];
    uint8_t num_buffer[5];

    memset(atcmd_buffer, 0, MAX_LEN_TRANSMIT);
    /* Joint the string for NWHTC command: "AT+NWHTC=URL,POST,data content\r\n" */
    strcat((char *)atcmd_buffer, (char *)"AT+NWHTC=http://");
    strcat((char *)atcmd_buffer, (char *)server_ip);
    strcat((char *)atcmd_buffer, (char *)":5000/mock,post,");
    strcat((char *)atcmd_buffer, (char *)data_to_send);
    strcat((char *)atcmd_buffer, (char *)" / number is ");
    sprintf((char *)num_buffer,"%d",num);
    strcat((char *)atcmd_buffer, (char *)num_buffer);
    strcat((char *)atcmd_buffer, (char *)"\r\n");
    g_da16200_cmd_set[DA16200_AT_CMD_INDEX_AT_NWHTC].p_cmd = atcmd_buffer;
    status = wifi_command_AT_NWHTC();
    if(status != FSP_SUCCESS)
    {
        return status;
    }
    return status;
}


